/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.mqtt;

import java.util.Map;
import rx.Observable;
import rx.Single;
import io.netty.handler.codec.mqtt.MqttQoS;
import io.netty.handler.codec.mqtt.MqttConnectReturnCode;
import io.vertx.rxjava.mqtt.messages.MqttSubscribeMessage;
import io.vertx.rxjava.mqtt.messages.MqttPublishMessage;
import io.vertx.rxjava.mqtt.messages.MqttUnsubscribeMessage;
import java.util.List;
import io.vertx.rxjava.core.buffer.Buffer;
import io.vertx.core.Handler;

/**
 * Represents an MQTT endpoint for point-to-point communication with the remote MQTT client
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.mqtt.MqttEndpoint original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rxjava.RxGen(io.vertx.mqtt.MqttEndpoint.class)
public class MqttEndpoint {

  public static final io.vertx.lang.rxjava.TypeArg<MqttEndpoint> __TYPE_ARG = new io.vertx.lang.rxjava.TypeArg<>(
    obj -> new MqttEndpoint((io.vertx.mqtt.MqttEndpoint) obj),
    MqttEndpoint::getDelegate
  );

  private final io.vertx.mqtt.MqttEndpoint delegate;
  
  public MqttEndpoint(io.vertx.mqtt.MqttEndpoint delegate) {
    this.delegate = delegate;
  }

  public io.vertx.mqtt.MqttEndpoint getDelegate() {
    return delegate;
  }

  /**
   * Close the endpoint, so the connection with remote MQTT client
   */
  public void close() { 
    delegate.close();
  }

  /**
   * @return the client identifier as provided by the remote MQTT client
   */
  public String clientIdentifier() { 
    if (cached_0 != null) {
      return cached_0;
    }
    String ret = delegate.clientIdentifier();
    cached_0 = ret;
    return ret;
  }

  /**
   * @return the Authentication information as provided by the remote MQTT client
   */
  public MqttAuth auth() { 
    if (cached_1 != null) {
      return cached_1;
    }
    MqttAuth ret = MqttAuth.newInstance(delegate.auth());
    cached_1 = ret;
    return ret;
  }

  /**
   * @return the Will information as provided by the remote MQTT client
   */
  public MqttWill will() { 
    if (cached_2 != null) {
      return cached_2;
    }
    MqttWill ret = MqttWill.newInstance(delegate.will());
    cached_2 = ret;
    return ret;
  }

  /**
   * @return the protocol version required by the remote MQTT client
   */
  public int protocolVersion() { 
    if (cached_3 != null) {
      return cached_3;
    }
    int ret = delegate.protocolVersion();
    cached_3 = ret;
    return ret;
  }

  /**
   * @return the protocol name provided by the remote MQTT client
   */
  public String protocolName() { 
    if (cached_4 != null) {
      return cached_4;
    }
    String ret = delegate.protocolName();
    cached_4 = ret;
    return ret;
  }

  /**
   * @return true when clean session is requested by the remote MQTT client
   */
  public boolean isCleanSession() { 
    if (cached_5 != null) {
      return cached_5;
    }
    boolean ret = delegate.isCleanSession();
    cached_5 = ret;
    return ret;
  }

  /**
   * @return the keep alive timeout (in seconds) specified by the remote MQTT client
   */
  public int keepAliveTimeSeconds() { 
    if (cached_6 != null) {
      return cached_6;
    }
    int ret = delegate.keepAliveTimeSeconds();
    cached_6 = ret;
    return ret;
  }

  /**
   * @return the message identifier used for last published message
   */
  public int lastMessageId() { 
    if (cached_7 != null) {
      return cached_7;
    }
    int ret = delegate.lastMessageId();
    cached_7 = ret;
    return ret;
  }

  /**
   * Enable/disable subscription/unsubscription requests auto acknowledge
   * @param isSubscriptionAutoAck auto acknowledge status
   */
  public void subscriptionAutoAck(boolean isSubscriptionAutoAck) { 
    delegate.subscriptionAutoAck(isSubscriptionAutoAck);
  }

  /**
   * @return true when auto acknowledge status for subscription/unsubscription requests
   */
  public boolean isSubscriptionAutoAck() { 
    boolean ret = delegate.isSubscriptionAutoAck();
    return ret;
  }

  /**
   * Enable/disable publishing (in/out) auto acknowledge
   * @param isPublishAutoAck auto acknowledge status
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishAutoAck(boolean isPublishAutoAck) { 
    delegate.publishAutoAck(isPublishAutoAck);
    return this;
  }

  /**
   * @return auto acknowledge status for publishing (in/out)
   */
  public boolean isPublishAutoAck() { 
    boolean ret = delegate.isPublishAutoAck();
    return ret;
  }

  /**
   * Enable/disable auto keep alive (sending ping response)
   * @param isAutoKeepAlive auto keep alive
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint autoKeepAlive(boolean isAutoKeepAlive) { 
    delegate.autoKeepAlive(isAutoKeepAlive);
    return this;
  }

  /**
   * @return the auto keep alive status (sending ping response)
   */
  public boolean isAutoKeepAlive() { 
    boolean ret = delegate.isAutoKeepAlive();
    return ret;
  }

  /**
   * @return if the connection between remote client and local endpoint is established/open
   */
  public boolean isConnected() { 
    boolean ret = delegate.isConnected();
    return ret;
  }

  /**
   * Set client identifier if not provided by the remote MQTT client (zero-bytes)
   * @param clientIdentifier the client identifier
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint setClientIdentifier(String clientIdentifier) { 
    delegate.setClientIdentifier(clientIdentifier);
    return this;
  }

  /**
   * Set a disconnect handler on the MQTT endpoint. This handler is called when a DISCONNECT
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint disconnectHandler(Handler<Void> handler) { 
    delegate.disconnectHandler(handler);
    return this;
  }

  /**
   * Set a subscribe handler on the MQTT endpoint. This handler is called when a SUBSCRIBE
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint subscribeHandler(Handler<MqttSubscribeMessage> handler) { 
    delegate.subscribeHandler(new Handler<io.vertx.mqtt.messages.MqttSubscribeMessage>() {
      public void handle(io.vertx.mqtt.messages.MqttSubscribeMessage event) {
        handler.handle(MqttSubscribeMessage.newInstance(event));
      }
    });
    return this;
  }

  /**
   * Set a unsubscribe handler on the MQTT endpoint. This handler is called when a UNSUBSCRIBE
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint unsubscribeHandler(Handler<MqttUnsubscribeMessage> handler) { 
    delegate.unsubscribeHandler(new Handler<io.vertx.mqtt.messages.MqttUnsubscribeMessage>() {
      public void handle(io.vertx.mqtt.messages.MqttUnsubscribeMessage event) {
        handler.handle(MqttUnsubscribeMessage.newInstance(event));
      }
    });
    return this;
  }

  /**
   * Set the publish handler on the MQTT endpoint. This handler is called when a PUBLISH
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishHandler(Handler<MqttPublishMessage> handler) { 
    delegate.publishHandler(new Handler<io.vertx.mqtt.messages.MqttPublishMessage>() {
      public void handle(io.vertx.mqtt.messages.MqttPublishMessage event) {
        handler.handle(MqttPublishMessage.newInstance(event));
      }
    });
    return this;
  }

  /**
   * Set the puback handler on the MQTT endpoint. This handler is called when a PUBACK
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishAcknowledgeHandler(Handler<Integer> handler) { 
    delegate.publishAcknowledgeHandler(handler);
    return this;
  }

  /**
   * Set the pubrec handler on the MQTT endpoint. This handler is called when a PUBREC
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishReceivedHandler(Handler<Integer> handler) { 
    delegate.publishReceivedHandler(handler);
    return this;
  }

  /**
   * Set the pubrel handler on the MQTT endpoint. This handler is called when a PUBREL
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishReleaseHandler(Handler<Integer> handler) { 
    delegate.publishReleaseHandler(handler);
    return this;
  }

  /**
   * Set the pubcomp handler on the MQTT endpoint. This handler is called when a PUBCOMP
   * message is received by the remote MQTT client
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishCompleteHandler(Handler<Integer> handler) { 
    delegate.publishCompleteHandler(handler);
    return this;
  }

  /**
   * Set the pingreq handler on the MQTT endpoint. This handler is called when a PINGREQ
   * message is received by the remote MQTT client. In any case the endpoint sends the
   * PINGRESP internally after executing this handler.
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint pingHandler(Handler<Void> handler) { 
    delegate.pingHandler(handler);
    return this;
  }

  /**
   * Set a close handler. This will be called when the MQTT endpoint is closed
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint closeHandler(Handler<Void> handler) { 
    delegate.closeHandler(handler);
    return this;
  }

  /**
   * Set an exception handler. This will be called when an error at protocol level happens
   * @param handler the handler
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint exceptionHandler(Handler<Throwable> handler) { 
    delegate.exceptionHandler(handler);
    return this;
  }

  /**
   * Sends the CONNACK message to the remote MQTT client with "connection accepted"
   * return code. See {@link io.vertx.rxjava.mqtt.MqttEndpoint#reject} for refusing connection
   * @param sessionPresent if a previous session is present
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint accept(boolean sessionPresent) { 
    delegate.accept(sessionPresent);
    return this;
  }

  /**
   * Sends the CONNACK message to the remote MQTT client rejecting the connection
   * request with specified return code. See {@link io.vertx.rxjava.mqtt.MqttEndpoint#accept} for accepting connection
   * @param returnCode the connect return code
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint reject(MqttConnectReturnCode returnCode) { 
    delegate.reject(returnCode);
    return this;
  }

  /**
   * Sends the SUBACK message to the remote MQTT client
   * @param subscribeMessageId identifier of the SUBSCRIBE message to acknowledge
   * @param grantedQoSLevels granted QoS levels for the requested topics
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint subscribeAcknowledge(int subscribeMessageId, List<MqttQoS> grantedQoSLevels) { 
    delegate.subscribeAcknowledge(subscribeMessageId, grantedQoSLevels);
    return this;
  }

  /**
   * Sends the UNSUBACK message to the remote MQTT client
   * @param unsubscribeMessageId identifier of the UNSUBSCRIBE message to acknowledge
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint unsubscribeAcknowledge(int unsubscribeMessageId) { 
    delegate.unsubscribeAcknowledge(unsubscribeMessageId);
    return this;
  }

  /**
   * Sends the PUBACK message to the remote MQTT client
   * @param publishMessageId identifier of the PUBLISH message to acknowledge
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishAcknowledge(int publishMessageId) { 
    delegate.publishAcknowledge(publishMessageId);
    return this;
  }

  /**
   * Sends the PUBREC message to the remote MQTT client
   * @param publishMessageId identifier of the PUBLISH message to acknowledge
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishReceived(int publishMessageId) { 
    delegate.publishReceived(publishMessageId);
    return this;
  }

  /**
   * Sends the PUBREL message to the remote MQTT client
   * @param publishMessageId identifier of the PUBLISH message to acknowledge
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishRelease(int publishMessageId) { 
    delegate.publishRelease(publishMessageId);
    return this;
  }

  /**
   * Sends the PUBCOMP message to the remote MQTT client
   * @param publishMessageId identifier of the PUBLISH message to acknowledge
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publishComplete(int publishMessageId) { 
    delegate.publishComplete(publishMessageId);
    return this;
  }

  /**
   * Sends the PUBLISH message to the remote MQTT client
   * @param topic topic on which the message is published
   * @param payload message payload
   * @param qosLevel quality of service level
   * @param isDup if the message is a duplicate
   * @param isRetain if the message needs to be retained
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint publish(String topic, Buffer payload, MqttQoS qosLevel, boolean isDup, boolean isRetain) { 
    delegate.publish(topic, payload.getDelegate(), qosLevel, isDup, isRetain);
    return this;
  }

  /**
   * Sends the PINGRESP message to the remote MQTT client
   * @return a reference to this, so the API can be used fluently
   */
  public MqttEndpoint pong() { 
    delegate.pong();
    return this;
  }

  private String cached_0;
  private MqttAuth cached_1;
  private MqttWill cached_2;
  private java.lang.Integer cached_3;
  private String cached_4;
  private java.lang.Boolean cached_5;
  private java.lang.Integer cached_6;
  private java.lang.Integer cached_7;

  public static  MqttEndpoint newInstance(io.vertx.mqtt.MqttEndpoint arg) {
    return arg != null ? new MqttEndpoint(arg) : null;
  }
}
